//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

// Load dts shapes and merge animations

exec("./wheelski_tire.cs");
exec("./wheelski_spring.cs");
exec("./wheelski.cs");

datablock TSShapeConstructor(WheelyDts)
{
	baseShape  = "./wheelchair.dts";
};    

datablock DebrisData( WheelyDebris )
{
   explodeOnMaxBounce = false;

   elasticity = 0.15;
   friction = 0.5;

   lifetime = 4.0;
   lifetimeVariance = 0.0;

   minSpinSpeed = 40;
   maxSpinSpeed = 600;

   numBounces = 5;
   bounceVariance = 0;

   staticOnMaxBounce = true;
   gravModifier = 1.0;

   useRadiusMass = true;
   baseRadius = 1;

   velocity = 20.0;
   velocityVariance = 12.0;
};             

datablock PlayerData(WheelyArmor : PlayerStandardArmor)
{
   renderFirstPerson = false;
   emap = false;
   
   className = Armor;
   shapeFile = "./wheelchair.dts";
   cameraMaxDist = 6;
   cameraTilt = 0.261;
   cameraVerticalOffset = 1.5;
     
   cameraDefaultFov = 90.0;
   cameraMinFov = 5.0;
   cameraMaxFov = 120.0;
   

   aiAvoidThis = true;

   mass = 90;
   drag = 0.1;
   density = 0.7;
   maxDamage = 250;
   maxEnergy =  10;
   repairRate = 0.33;

   rechargeRate = 0.4;

   runEnergyDrain = 0;
   minRunEnergy = 0;
   maxSideSpeed = 0;

   maxSideCrouchSpeed = 0;
	
	maxForwardCrouchSpeed = 7;
	maxBackwardCrouchSpeed = 4;
	
   maxForwardProneSpeed = 0;
   maxBackwardProneSpeed = 0;
   maxSideProneSpeed = 0;

   maxForwardWalkSpeed = 0;
   maxBackwardWalkSpeed = 0;
   maxSideWalkSpeed = 0;

   maxUnderwaterForwardSpeed = 8.4;
   maxUnderwaterBackwardSpeed = 7.8;
   maxUnderwaterSideSpeed = 7.8;

   jumpEnergyDrain = 0;
   minJumpEnergy = 0;
   jumpDelay = 0;

   minJetEnergy = 0;
	jetEnergyDrain = 0;
	canJet = 0;

   minImpactSpeed = 250;
   speedDamageScale = 3.8;

   boundingBox			= vectorScale("2.2 2.2 2.4", 4);
   crouchBoundingBox	= vectorScale("2.2 2.2 2.4", 4);
   
   pickupRadius = 0.75;
   
   // Foot Prints
   //decalOffset = 0.25;
	
   jetEmitter = "";
   jetGroundEmitter = "";
   jetGroundDistance = 4;
  
   //footPuffEmitter = LightPuffEmitter;
   footPuffNumParts = 10;
   footPuffRadius = 0.25;

   //dustEmitter = LiftoffDustEmitter;

   splash = PlayerSplash;
   splashVelocity = 4.0;
   splashAngle = 67.0;
   splashFreqMod = 300.0;
   splashVelEpsilon = 0.60;
   bubbleEmitTime = 0.1;
   splashEmitter[0] = PlayerFoamDropletsEmitter;
   splashEmitter[1] = PlayerFoamEmitter;
   splashEmitter[2] = PlayerBubbleEmitter;
   mediumSplashSoundVelocity = 10.0;   
   hardSplashSoundVelocity = 20.0;   
   exitSplashSoundVelocity = 5.0;

   // Controls over slope of runnable/jumpable surfaces
   runSurfaceAngle  = 85;
   jumpSurfaceAngle = 86;

   minJumpSpeed = 20;
   maxJumpSpeed = 30;

   horizMaxSpeed = 68;
   horizResistSpeed = 33;
   horizResistFactor = 0.35;

   upMaxSpeed = 80;
   upResistSpeed = 25;
   upResistFactor = 0.3;
   
   footstepSplashHeight = 0.35;

   //NOTE:  some sounds commented out until wav's are available

   JumpSound			= HorseJumpSound;
   
   impactWaterEasy      = Splash1Sound;
   impactWaterMedium    = Splash1Sound;
   impactWaterHard      = Splash1Sound;
   
   groundImpactMinSpeed    = 10.0;
   groundImpactShakeFreq   = "4.0 4.0 4.0";
   groundImpactShakeAmp    = "1.0 1.0 1.0";
   groundImpactShakeDuration = 0.8;
   groundImpactShakeFalloff = 10.0;
   
   //exitingWater         = ExitingWaterLightSound;

   // Inventory Items
	maxItems   = 10;	//total number of bricks you can carry
	maxWeapons = 5;		//this will be controlled by mini-game code
	maxTools = 5;
	
	uiName = "Wheelchair";
	rideable = true;
	lookUpLimit = 0.5;
	lookDownLimit = 1;

	canRide = true;
	showEnergyBar = false;
	paintable = true;

   numMountPoints = 1;
   mountNode[0] = 6;
   mountThread[0] = "sit";
   mountThread[6] = "sit"; 
};

package WheelyColor
{
	function GameConnection::applyBodyColors(%client)
	{
		Parent::applyBodyColors(%client);
		if(%client.player.dataBlock $= "WheelyArmor")
		{
			
			%color1 = "0.45 0.45 0.45 1";
			%color2 = "0.21 0.21 0.21 1";
			%color3 = "0.30 0.30 0.30 1";
			%client.player.setNodeColor("chairRhand",%client.rHandColor);
			%client.player.setNodeColor("chairLhand",%client.lHandColor);
			%client.player.setNodeColor("chairLarm",%client.lArmColor);
			%client.player.setNodeColor("chairRarm",%client.rArmColor);
			%client.player.setNodeColor("chairLfoot",%client.lLegColor);
			%client.player.setNodeColor("chairRfoot",%client.rLegColor);
			%client.player.setNodeColor("chairBody",%client.chestColor);
			%client.player.setNodeColor("chairPants",%client.hipColor);
			%client.player.setNodeColor("chairFrame",%color1);
			%client.player.setNodeColor("chairHandle",%color2);
			%client.player.setNodeColor("RWheel",%color2);
			%client.player.setNodeColor("LWheel",%color2);
			%client.player.setNodeColor("RRStock",%color2);
			%client.player.setNodeColor("LRStock",%color2);
			%client.player.setNodeColor("RSWheel",%color3);
			%client.player.setNodeColor("LSWheel",%color3);
			%client.player.setNodeColor("RBWheel",%color3);
			%client.player.setNodeColor("LBWheel",%color3);
			%client.player.HideNode(RRMoney);
			%client.player.HideNode(LRMoney);
			%obj = %client.player;
			scheduleNoQuota(100,%obj,delayWheelyCheck,%obj);
		}
	}
	
	function ShapeBase::setNodeColor(%obj,%node,%color)
    {
		Parent::setNodeColor(%obj,%node,%color);
		
		//torque has some dumb preset garbage and one of them converts any case of "all" to "All" and node functions are case sensitive
		%bullshit = getSubstr(%node,0,1) @ getSubstr(%node,1,1) @ getSubstr(%node,2,1);
		
		if(%bullshit $= "ALL" && %obj.getdataBlock().getname() $= "WheelyArmor")
    	{
			WheelyVehicleColors(%obj,%color);
    	}
    }
	
	function SkiWeaponImage::onFire(%this, %obj, %slot)
	{
		
		%player = %obj;

		if(%player.isMounted())
		{
			
			%mountedVehicleName = %player.getObjectMount().getDataBlock().getName();
			if(%mountedVehicleName $= "WheelyArmor")
			{	
				
				%wheelchair = %player.getObjectMount();
				if(%wheelchair.isMounted())
				{
					
					if(%wheelchair.getObjectMount().getDataBlock().getName() $= "WheelyskiVehicle")
					{	
						//we're mounted on a WheelyskiVehicle, so stop skiing
						%wheelchair.stopWheelySkiing();
						%wheelchair.unMount();
						return;
					}
					else
					{
						//they're on a vehicle that is not either a wheelchair or a drift wheelchair
						commandToClient(%player.client, 'CenterPrint', "\c4Can\'t use skis right now.", 2);
						return;
					}

				}
				else
				{
				%player.startWheelySkiing(%wheelchair);
				commandToClient(%player.client,'setScrollMode', -1);
				%player.unMountimage(%slot);
				fixArmReady(%player);
				return;
				}
				
			}
			else if(%mountedVehicleName $= "WheelyskiVehicle")
			{
				//we're mounted on a WheelyskiVehicle, so stop skiing
				%player.stopWheelySkiing();
				%player.unMount();
				return;
			}
		}
		else if(%player.getDataBlock().getName() $= "WheelyArmor")
		{
			%wheelchair = 0;
			%player.startWheelySkiing(%wheelchair);
			commandToClient(%player.client,'setScrollMode', -1);
			%player.unMountimage(%slot);
			fixArmReady(%player);
			return;
		}
		
		Parent::onFire(%this, %obj, %slot);
		
	}
	
	function Armor::onUnMount(%this,%obj,%vehicle,%node)
	{
		Parent::onUnMount(%this,%obj,%vehicle,%node);
		%veh = %vehicle;
		if(isObject(%veh) && %veh.isMounted())
		{
			%mountedvehicle = %veh.getObjectMount().getDataBlock().getName();
			if(%mountedvehicle $= "WheelyskiVehicle")
			{
				%veh.unMount();
			}
		}
		
	}
	
};
activatePackage(WheelyColor);

function WheelyArmor::onAdd(%this,%obj)
{
   // Vehicle timeout
	%obj.mountVehicle = true;
	%obj.setRepairRate(0);
	%obj.HideNode(RRMoney);
	%obj.HideNode(LRMoney);
	
	scheduleNoQuota(200,%obj,delayWheelyCheck,%obj);
   
}

function WheelyVehicleColors(%obj,%color)
{
	%color0	= vectorScale(%color,"0.8 0.8 0.8 1") SPC 1;
	%color1 = "0.45 0.45 0.45 1";
	%color2 = "0.21 0.21 0.21 1";
	%color3 = "0.30 0.30 0.30 1";
	%obj.setNodeColor("chairRhand",%color);
	%obj.setNodeColor("chairLhand",%color);
	%obj.setNodeColor("chairLarm",%color0);
	%obj.setNodeColor("chairRarm",%color0);
	%obj.setNodeColor("chairLfoot",%color0);
	%obj.setNodeColor("chairRfoot",%color0);
	%obj.setNodeColor("chairBody",%color);
	%obj.setNodeColor("chairPants",%color);
	%obj.setNodeColor("chairFrame",%color1);
	%obj.setNodeColor("chairHandle",%color2);
	%obj.setNodeColor("RWheel",%color2);
	%obj.setNodeColor("LWheel",%color2);
	%obj.setNodeColor("RRStock",%color2);
	%obj.setNodeColor("LRStock",%color2);
	%obj.setNodeColor("RSWheel",%color3);
	%obj.setNodeColor("LSWheel",%color3);
	%obj.setNodeColor("RBWheel",%color3);
	%obj.setNodeColor("LBWheel",%color3);
	
}


function delayWheelyCheck(%obj)
{
	if(isObject(%obj))
	{
		if(%obj.name $= "Mad Money")
		{
			%obj.unHideNode(RRMoney);
			%obj.unHideNode(LRMoney);
			%obj.HideNode(RRStock);
			%obj.HideNode(LRStock);
			%obj.name = "Mad Money";
			%obj.setNodeColor("RRMoney","1 0.9 0 1");
			%obj.setNodeColor("LRMoney","1 0.9 0 1");
		}
		else
		{
		%obj.HideNode(RRMoney);
		%obj.HideNode(LRMoney);
		%obj.unHideNode(RRStock);
		%obj.unHideNode(LRStock);	
		}
	}
}

//called when the driver of a player-vehicle is unmounted
function WheelyArmor::onDriverLeave(%this, %obj, %player)
{

}

function Player::startWheelySkiing(%obj, %wheelchair)
{	
	//make a new ski vehicle and mount the player on it
	%client = %obj.client;
	%position = %obj.getTransform();
	%posX = getword(%position, 0);
	%posY = getword(%position, 1);
	%posZ = getword(%position, 2);
	%rot = getWords(%position, 3, 8);

	%posZ += 0.3;
	
	%vel = %obj.getVelocity();

	%newcar = new WheeledVehicle() 
	{
		dataBlock = Wheelyskivehicle;
		client = %client;
		initialPosition = %posX @ " " @ %posY @ " " @ %posZ;
	};
   MissionCleanup.add(%newcar);
	%newcar.setVelocity(%vel);
	%newcar.setTransform(%posX @ " " @ %posY @ " " @ %posZ @ " " @ %rot);
	

	
	
	//If the Wheelchair is a player, or if the player is mounted on a wheelchair we have to figure out which one turns into a Drifting Wheelchair
	if(!%wheelchair)
	{
		%newcar.schedule(250, mountObject, %obj, 0);
		%obj.hidenode(RBWheel);
		%obj.hidenode(LBWheel);
		%obj.hidenode(RRMoney);
		%obj.hidenode(LRMoney);
		%obj.hidenode(RRStock);
		%obj.hidenode(LRStock);
		%newcar.setscale(%obj.scale);
	}
	else
	{
		%newcar.schedule(250, mountObject, %wheelchair, 0);
		%wheelchair.hidenode(RBWheel);
		%wheelchair.hidenode(LBWheel);
		%wheelchair.hidenode(RRMoney);
		%wheelchair.hidenode(LRMoney);
		%wheelchair.hidenode(RRStock);
		%wheelchair.hidenode(LRStock);
		%newcar.setscale(%wheelchair.scale);
		if(%wheelchair.name $= "Mad Money")
		{
			%newcar.setWheelTire(2,WheelySkiRMTire);
			%newcar.setWheelTire(3,WheelySkiRMTire);
		}
	}
	
}

function Player::stopWheelySkiing(%obj)
{

	if(%obj.getDataBlock().getName() $= "WheelyArmor")
	{
		
		%obj.unhidenode(RBWheel);
		%obj.unhidenode(LBWheel);
		delayWheelyCheck(%obj);
		
	}
	

	return;


	//VVV super old inventory stuff VVV

	//de-equip-hilight inv slot
	%player = %obj;
	%playerData = %player.getDataBlock();
	%client = %player.client;
	for(%i = 0; %i < %playerData.maxItems; %i++)							//search through other inv slots
	{
		if(%player.isEquiped[%i] == true)									//if it is equipped then
		{
			if(%player.inventory[%i] == ski.getId())								//if it is skis
			{		
				messageClient(%client, 'MsgDeEquipInv', '', %i);			//then de-equip it 
				%player.isEquiped[%i] = false;
				break;														//we're done because only one item can interfere
			}										
		}
	}

}
